YARP 3.5.1 (2021-11-10)                                                {#v3_5_1}
=======================

[TOC]

YARP 3.5.1 Release Notes
========================


A (partial) list of bug fixed and issues resolved in this release can be found
[here](https://github.com/robotology/yarp/issues?q=label%3A%22Fixed+in%3A+YARP+v3.5.1%22).


New Features
------------

### Libraries

#### `math`

* added method `yarp::math::Quaternion::isValid()`
* added method `yarp::math::FrameTransform::isValid()`



### Build System

* Fixed infinite loop when using yarp_add_idl and yarp_idl_to_dir without
  searching the `idl_tools` component.
* Fixed build on Ubuntu Bionic when both PCL and OpenNI are enabled


### Libraries

#### `os`

##### `Log`

* Fixed custom log components not forwarding the log properly when running under
  `yarprun`.

##### `ResourceFinder`

* Fixed folder creation in `getHomeContextPath` and `getHomeRobotPath`.

##### `idl::WireReader`

* Fixed reading empty string as latest argument.


### `dev`

* Fixed parsing of parameters `hw_gain` and `sw_gain` in the two classes
  `AudioPlayerDeviceBase` and `AudioRecorderDeviceBase`

* Fixed issue with case insensitive file systems.

* Added methods `startStorageThread()` and `stopStorageThread()` to interface
 `yarp::dev::IFrameTransformStorageUtils`



### `robotinterface`

* Fixed `extern-name` for non-string parameters.

### `run`

* Fixed handling of multiple environment variables with `yarprun` on win32


### Devices

#### `audioFromFileDevice`

* Fixed module when channels are > 1

#### `controlBoard_nws_yarp`

* Fixed segmentation fault if the device to which the nws attempts to attach was
  not successfully opened.

### `controlBoard_nws_ros`

* `/joint_states` is no longer appended to the topic name.

#### `controlboardremapper`

* Fixed segmentation fault.

#### `ffmpeg_grabber`

* Fixed symbol lookup runtime error.

#### `ffmpeg_writer`

* Fixed symbol lookup runtime error.

### `frameGrabber_nws_ros`

* The `frame_id` parameter is no longer required to start with `/`.

#### `frameTransformClient`

* Fixed concurrent access to the internal container.
* Timestamp is now set also for static_transforms
* Updated xml files according to the changes to the subdevices.

#### `frameTransformServer`

* Updated xml files according to the changes to the subdevices.

#### `frameTransformGet_nwc_yarp`

* Fixed missing callback initialization. Without it, no data was collected from
  the streaming port.

#### `frameTransformSetMultiplexer`

* fixed race condition issue in `deleteTransform()` method.

#### `frameTransformStorage`

* Fixed race condition issue: added extra mutex to protect the periodicThread
  from set/get/delete operations
* the periodic thread can be now stopped/started on request by
  methods `IFrameTransformStorageUtils::stopStorageThread` and
  `IFrameTransformStorageUtils::startStorageThread`.
* Fixed `start` method not called after successfully attaching to a
  `yarp::dev::IFrameTransformStorageGet` interface.
  The device is now able to update its `FrameTransformContainer` storage by
  querying the device it was attached to.
* Fixed attach function that did not performed the view on the
  `iFrameTransformStorageGet` interface and did not return `true` when
  successful.

#### `frameTransformContainer`

* Added new iterator class. It iterates only on valid transforms.
* when a transform is deleted, it is marked as invalid. The transform is then
  removed during the next iteration of the `checkAndRemoveExpired()`.
* Fixed bug in `frameTransformContainer` which was not correctly deleting old
  transforms.

#### `frameTransformSet_nwc_yarp`

* The device was refactored in order to easily manage ports names.

#### `frameTransformSet_nws_yarp`

* The device was refactored in order to easily manage ports names.

#### `frameTransformGet_nwc_yarp`

* The device was refactored in order to easily manage ports names.

#### `frameTransformGet_nws_yarp`

* The device was refactored in order to easily manage ports names.

#### `frameTransformSet_nwc_ros`

* Fixed bug in `frameTransformSet_nwc_ros::publishFrameTransforms`.

#### `frameTransformSet_nws_ros`

* Fixed time stamps of the static transforms wrongly set to
 `yarp::os::Time::now()`.

#### `frameTransformGet_nwc_ros`

* Fixed `setStrict` not called for the subscribers ports for the `/tf` and
  `/tf_static` topics, that was causing a loss of frameTransforms.

#### `map2D_nws_yarp`

* Fixed typos in `Map2D_nws_yarp::parse_vocab_command`.

#### `opencv_grabber`

* Efficiency has been improved as copies are (in most cases) no longer
  performed in order to convert from OpenCV frames to YARP image structures.
* Options `--flip_x`, `--flip_y` and `--transpose` are now also available
  when reading from file (`--movie`).

#### `rgbdSensor_nws_ros`

* Fixed bug in `rgbdSensor_nws::open`. The period was still in milliseconds.

#### `transformClient`

* Fixed method delete in `transformClient`, now avoiding race conditions during
  server/client update via streaming port.


### Port Monitors

#### `image_compression_ffmpeg`

* Fixed symbol lookup runtime error.


### GUIs

#### `yarplogger`

* Fixed bug.

#### `yarpmanager`

* Port names containing trailing or leading white spaces not targeted as
  `non-existing` but throwing an error when trying to connect.

### Bindings

#### Python

* Enable compilation of `libYARP_math` in the Python wheels

Contributors
------------

This is a list of people that contributed to this release (generated from the
git history using `git shortlog -ens --no-merges v3.5.0..v3.5.1`):

```
    26	Daniele E. Domenichelli <daniele.domenichelli@iit.it>
    23	Marco Randazzo <marco.randazzo@iit.it>
    20	Ettore Landini <ettore.landini@iit.it>
     6	Diego Ferigo <diego.ferigo@iit.it>
     2	Bartek Łukawski <bwmn.peter@gmail.com>
     1	Marco Randazzo <marco.randaz@iit.it>
     1	Silvio Traversaro <silvio.traversaro@iit.it>
     1	Stefano Bernagozzi <stefano.bernagozzi@iit.it>
     1	Ugo Pattacini <ugo.pattacini@iit.it>
```
